
C-{{{ subroutine slhablocks:

      subroutine slhablocks(blocktype,cline,ierr)
c..
c..   Read the data in string CLINE according to the format
c..   of block BLOCKTYPE.
c..   Note that BLOCKTYPE must ALWAYS be in capital letters, and
c..   of length *15 (trailing blanks).
c..
c..   Here the COMMON blocks are actually filled with data.
c..
c..   Example:
c..   BLOCKTYPE = 'MASS           '
c..   CLINE = ' 25  1.14272938e02  '
c..
c..   Then smassbo(25) would be given the value 1.14272938e02.
c..   In other words, M_h = 114 GeV.
c..   For the particle codes, see SUBROUTINE SLHAMAP.
c..
c..   Note that no checking of the correct format of CLINE is done.
c..   
      implicit real*8(a-h,o-z)
      integer pdgcode
      real*8 rval
      character blocktype*15,cline*200,rchar*80
      include 'common-slha.f'

      ierr=0

      if (blocktype.eq.'MASS          ') then
         ifound = 1
         read(cline,*,ERR=510) i,rval
         if (i.le.100) then
            smassbo(i) = rval
         elseif (i.lt.2000000) then
            smassfe1(i-1000000) = rval
         else 
            smassfe2(i-2000000) = rval
         endif
      endif
      
      if (blocktype.eq.'SMINPUTS      ') then
         read(cline,*,ERR=510) i,rval
         sminputs(i) = rval
      endif

      if (blocktype.eq.'STOPMIX       ') then
         read(cline,*,ERR=510) i,j,rval
         stopmix(i,j) = rval
      endif

      if (blocktype.eq.'MINPAR        ') then
         read(cline,*,ERR=510) i,rval
         minpar(i) = rval
      endif

      if (blocktype.eq.'ALPHA         ') then
         read(cline,*,ERR=510) rval
         slhaalpha = rval
      endif
      
      if (blocktype.eq.'HMIX          ') then
         read(cline,*,ERR=510) i,rval
         hmix(i) = rval
      endif

      if (blocktype.eq.'GAUGE         ') then
         read(cline,*,ERR=510) i,rval
         gauge(i) = rval
      endif

      if (blocktype.eq.'CREIN         ') then
         read(cline,*,ERR=510) i
         if ((i.eq.1).or.(i.eq.2)) then
            read(cline,*,ERR=510) i,rchar
            creinchar(i) = rchar
         elseif (i.le.5) then
            read(cline,*,ERR=510) i,j
            creinvar(i) = j
         elseif (i.le.100) then
            read(cline,*,ERR=510) i,rval
            creinvar2(i) = rval
            lcreinvar2(i) = .true.
         else
            read(cline,*,ERR=510) i,rval
            if (i.lt.2000) then
               creinminpar(1,i-1000) = rval
            elseif (i.lt.3000) then
               creinminpar(2,i-2000) = rval
            elseif (i.lt.4000) then
               creinminpar(3,i-3000) = rval
            elseif (i.lt.5000) then
               creinminpar(4,i-4000) = rval
            else 
               creinminpar(5,i-5000) = rval
            endif
         endif
      endif
      
      return

 510  ierr=1
      
      end

C-}}}
C-{{{ subroutine slhamap:

      subroutine slhamap
c..
c..   Map the SLHA identification codes to names.
c..   For example: 
c..   csmassbo(25) = 'M(h)'
c..   i.e., smassbo(25) contains the mass of the light Higgs.
c..   
      include 'common-slha.f'

c..   fill with empty string:
      do i=1,100
         write(csminputs(i),3001)
         write(csmassbo(i),3001)
         write(csmassfe1(i),3001)
         write(csmassfe2(i),3001)
         write(cminpar(i),3001)
         write(chmix(i),3001)
         write(cgauge(i),3001)
      enddo
      write(cslhaalpha,3001)

      csminputs(1) = '1/alphaQED(Mz)'
      csminputs(2) = 'GFermi'
      csminputs(3) = 'alpha_s(Mz)'
      csminputs(4) = 'M(Z)'
      csminputs(5) = 'mb(mb)'
      csminputs(6) = 'M(top)'
      csminputs(7) = 'M(tau)'

      csmassbo(25) = 'M(h)'
      csmassbo(35) = 'M(H)'
      csmassbo(36) = 'M(A)'
      csmassbo(37) = 'M(H+)'

      csmassfe1(1) = 'M(d_L)'
      csmassfe1(2) = 'M(u_L)'
      csmassfe1(3) = 'M(s_L)'
      csmassfe1(4) = 'M(c_L)'
      csmassfe1(5) = 'M(b_1)'
      csmassfe1(6) = 'M(t_1)'

      csmassfe2(1) = 'M(d_R)'
      csmassfe2(2) = 'M(u_R)'
      csmassfe2(3) = 'M(s_R)'
      csmassfe2(4) = 'M(c_R)'
      csmassfe2(5) = 'M(b_2)'
      csmassfe2(6) = 'M(t_2)'

      cminpar(1) = 'm0'
      cminpar(2) = 'm12'
      cminpar(3) = 'tanb'
      cminpar(4) = 'sign(mu)'
      cminpar(5) = 'A0'

      cslhaalpha = 'alpha'

      chmix(1) = 'mu(Q)'

      cgauge(1) = 'gprime(Q)^DRbar'
      cgauge(2) = 'g(Q)^DRbar'
      cgauge(3) = 'g3(Q)^DRbar'

      ccreinchar(1) = 'jdir      '
      ccreinchar(2) = 'jfname    '
      ccreinvar(3) = 'MODSEL'
      ccreinvar(4) = 'inumber'
      ccreinvar(5) = 'ivary'
      ccreinvar(6) = 'q0'
      ccreinvar(7) = 'mu'
      ccreinminpar(1,1) = 'MINPARMIN1'
      ccreinminpar(1,2) = 'MINPARMAX1'
      ccreinminpar(1,3) = 'MINPARSTEP1'
      ccreinminpar(2,1) = 'MINPARMIN2'
      ccreinminpar(2,2) = 'MINPARMAX2'
      ccreinminpar(2,3) = 'MINPARSTEP2'
      ccreinminpar(3,1) = 'MINPARMIN3'
      ccreinminpar(3,2) = 'MINPARMAX3'
      ccreinminpar(3,3) = 'MINPARSTEP3'
      ccreinminpar(4,1) = 'MINPARMIN4'
      ccreinminpar(4,2) = 'MINPARMAX4'
      ccreinminpar(4,3) = 'MINPARSTEP4'
      ccreinminpar(5,1) = 'MINPARMIN5'
      ccreinminpar(5,2) = 'MINPARMAX5'
      ccreinminpar(5,3) = 'MINPARSTEP5'

 3001 format(20a)
      
      end

C-}}}
